/**
 * Config responsible for building Walkontable (bundled into `src/3rdparty/walkontable/dist/`):
 *  - walkontable.js
 */
const path = require('path');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const compilationDoneMarker = require('./plugin/webpack/compilation-done-marker');

const wotPath = path.resolve(__dirname, '../src/3rdparty/walkontable');

module.exports.create = function create() {
  const config = {
    devtool: 'cheap-module-source-map',
    mode: 'none',
    output: {
      filename: 'walkontable.js',
      globalObject: `typeof self !== 'undefined' ? self : this`,
      library: 'Walkontable',
      libraryTarget: 'var',
      path: path.resolve(wotPath, 'dist'),
    },
    module: {
      rules: [
        {
          test: /\.js$/,
          loader: 'babel-loader',
          exclude: [
            /node_modules/,
          ],
          options: {
            cacheDirectory: true,
          },
        },
        {
          test: /\.(scss|css)$/,
          use: [
            { loader: MiniCssExtractPlugin.loader },
            { loader: 'css-loader' },
            { loader: 'sass-loader'},
            { loader: path.resolve(__dirname, 'loader/sass-rtl-loader.js')}
          ]
        },
      ]
    },
    plugins: [
      new MiniCssExtractPlugin({ filename: 'walkontable.css' }),
      compilationDoneMarker(),
    ],
  };
  return [].concat(config);
}
